# Visualization creation

## Grafana

Grafana is a visualization tool targeted for monitoring. It allows us to query, visualize, and define alerts based on data from different data sources. Currently, in the [monit-grafana](http://monit-grafana.cern.ch/) CMS organization, we have [ElasticSearch](https://www.elastic.co/what-is/elasticsearch), [Prometheus](https://prometheus.io/), [InfluxDB](https://www.influxdata.com/), and [Graphite](https://graphiteapp.org/) data sources. 

### How can I access my data using Grafana?

To use the data a new Grafana data source need to be defined. When you have a MonIT flow which saves data to InfluxDB, the MonIT team will define a new Grafana data source for you, it will have the monit_idb prefix in the name.  Similarly, to access data either in ElasticSearch, Prometheus or Graphite, either the MonIT team or the CMS Monitoring team can define a data source for you. 

Once you have a Grafana data source you can test it either in the explore tab or creating a new dashboard.  

### How to create a Grafana Dashboard?

All users in the CMS Grafana organization can create dashboards and save them at the playground folder. Before creating the first dashboard, let's take note of some recommendations:



- Grafana dashboards are defined as JSON documents. Sometimes the best way to debug a problem is to look at the panel or dashboard JSON definition. 
- Filters are defined as template variables. 
- Queries are defined in the language of the data source. Grafana is only a client to visualize the data, if you have a problem with the query, look at the data source query language documentation.
- Grafana documentation and issues are a good point to start searching for specific topics.  [Grafana playground](https://play.grafana.org/) is a good place to find inspiration and make experiments. 

Let's create our first dashboard based on the data published in the previous tutorial. While reading the following steps, you can view the video at https://youtu.be/M1uS40urq5I (each step is linked to the relevant video position).

1. Create a [new dashboard](https://www.youtube.com/watch?v=M1uS40urq5I).

   To create a new dashboard, you need to login to https://monit-grafana.cern.ch using the CERN SSO.  Then, you can use the `new` icon (+) and select dashboard. 

2. Create a [new panel using a query to the ES source](https://youtu.be/M1uS40urq5I?t=12) and using the visualization tab to make it a table. 

   When a new dashboard is created you can add a new panel. You can start selecting the query or selecting the visualization type. This time we will start with the query. Next, we need to select the data source, in this case, we will use the training dataset `monit_prod_cms-training`.

   Then we need to create the query using the graphical editor. For now, we will query for all documents, and we want to show the raw JSON documents. 

   Once we have the query, we can use the visualization tab to select the visualization type we want(table).

   Now we have our first panel. To add another panel, you can use the add icon ![add icon](https://raw.githubusercontent.com/grafana/grafana/master/public/img/icons_light_theme/icon_add_panel.svg?sanitize=true)

3. Create a [new template variable](https://youtu.be/M1uS40urq5I?t=92) and modify the query to use the new filter. 
   We can create a new variable based on a query to a dataset. In this case, we want to have a variable to select the user who has sent the data. To do that we can use the following ES query:

   ```json
   {"find": "terms", "field": "data.training_username", "size": 10000}
   ```

   To be able to select more than one user at the time we will select the multi-value option. Also, we'll select the `Include All option` option. If we don't introduce a custom all value, Grafana will select all the values (making a long query string if the variable has too many values).

   Once we have verified the values, we need to use the `add` button to create the variable. 

   To use the variable, we need to add it to the panel's queries. To learn more about the template variables, you can read the [grafana documentation](https://grafana.com/docs/grafana/latest/reference/templating/).

4. Create a new [line plot based on the same query](https://youtu.be/M1uS40urq5I?t=208).

   Now we will duplicate the existing panel and modify it to have a line plot. To do so we need to select the aggregation and the metric we want. 

   Note that in the time histogram aggregation we select an interval equal to the interval we send the data. 

5. Create a new [pie chart](https://youtu.be/M1uS40urq5I?t=340). 
   To create a pie chart is similar to create a line plot. In this case, in the visualization tab, we will change the value to show the total (sum of the values for the period), instead of the current(last) value. 

## Kibana

Kibana is a visualization tool for ElasticSearch. It is useful especially for explore the ES indexes. Using [monit-kibana](https://monit-kibana.cern.ch/kibana/app/kibana#/discover), you can select the `monit_prod_cms-training_raw_metric_*` index pattern in the discover tab. Using the left panel, you can select the relevant fields. You can use the query bar or the filters to select the relevant documents.  